@@ -10,6 +10,7 @@ class FluentStructure(object):
         self.external_interface_name = None
         self.external_new_methods = []
         self.external_rpc_methods = []
+        self.internal_http_methods = []
         self.components = {}
 
 
@@ -21,6 +22,7 @@ class FluentRpc(object):
         self.method = None
         self.request = "emptypb.Empty{}"
         self.responses = []
+        self.http_call = None
 
 
 class FluentRpcResponse(object):
@@ -32,6 +34,18 @@ class FluentRpcResponse(object):
     def __init__(self):
         self.status_code = None
         self.schema = None
+        self.request_return_type = None
+
+
+class FluentHttp(object):
+    """httpSetConfig(config Config) error"""
+
+    def __init__(self):
+        self.operation_name = None
+        self.method = None
+        self.request = None
+        self.request_return_type = None
+        self.responses = []
 
 
 class FluentNew(object):
@@ -44,6 +58,7 @@ class FluentNew(object):
         self.method = None
         self.schema_name = None
         self.schema_object = None
+        self.isRpcResponse = False
         self.interface_fields = []
 
     def isOptional(self, property_name):
@@ -164,7 +179,6 @@ class OpenApiArtGo(OpenApiArtPlugin):
         self._build_api_interface()
         self._build_request_interfaces()
         self._write_component_interfaces()
-        self._write_response_interfaces()
         self._close_fp()
 
     def _write_package_docstring(self, info_object):
@@ -188,6 +202,9 @@ class OpenApiArtGo(OpenApiArtPlugin):
         self._write(line)
         self._write('import "google.golang.org/protobuf/types/known/emptypb"')
         self._write('import "google.golang.org/grpc"')
+        self._write('import "github.com/ghodss/yaml"')
+        self._write('import "google.golang.org/protobuf/encoding/protojson"')
+        self._write('import "github.com/golang/protobuf/proto"')
         with open(os.path.join(os.path.dirname(__file__), "common.go")) as fp:
             self._write(fp.read().strip().strip("\n"))
         self._write()
@@ -200,7 +217,10 @@ class OpenApiArtGo(OpenApiArtPlugin):
 
     def _get_internal_name(self, openapi_name):
         name = self._get_external_struct_name(openapi_name)
-        return name[0].lower() + name[1:]
+        name = name[0].lower() + name[1:]
+        if name in ["error"]:
+            name = "_" + name
+        return name
 
     def _get_external_field_name(self, openapi_name):
         external_name = ""
@@ -227,51 +247,92 @@ class OpenApiArtGo(OpenApiArtPlugin):
     def _build_api_interface(self):
         self._api.internal_struct_name = """{internal_name}Api""".format(internal_name=self._get_internal_name(self._go_sdk_package_name))
         self._api.external_interface_name = """{external_name}Api""".format(external_name=self._get_external_struct_name(self._go_sdk_package_name))
-        for operation_id in self._get_parser("$.paths..operationId").find(self._openapi):
-            path_item_object = operation_id.context.value
-            rpc = FluentRpc()
-            rpc.operation_name = self._get_external_struct_name(operation_id.value)
-            if len([m for m in self._api.external_rpc_methods if m.operation_name == rpc.operation_name]) == 0:
-                self._api.external_rpc_methods.append(rpc)
-            ref = self._get_parser("$..requestBody..'$ref'").find(path_item_object)
-            if len(ref) == 1:
-                new = FluentNew()
-                new.schema_name = self._get_schema_object_name_from_ref(ref[0].value)
-                new.schema_object = self._get_schema_object_from_ref(ref[0].value)
-                new.interface = self._get_external_struct_name(new.schema_name)
-                new.struct = self._get_internal_name(new.schema_name)
-                new.method = """New{interface}() {interface}""".format(interface=new.interface)
-                if len([m for m in self._api.external_new_methods if m.schema_name == new.schema_name]) == 0:
-                    self._api.external_new_methods.append(new)
-                rpc.request = "{pb_pkg_name}.{operation_name}Request{{{interface}: {struct}.msg()}}".format(
-                    pb_pkg_name=self._protobuf_package_name,
-                    operation_name=rpc.operation_name,
-                    interface=new.interface,
-                    struct=new.struct,
-                )
-                rpc.method = """{operation_name}({struct} {interface}) ({operation_response_name}Response_StatusCode200, error)""".format(
-                    operation_name=rpc.operation_name,
-                    operation_response_name=self._get_external_struct_name(rpc.operation_name),
-                    struct=new.struct,
-                    interface=new.interface,
-                )
-            else:
-                rpc.method = """{operation_name}() ({operation_response_name}Response_StatusCode200, error)""".format(
-                    operation_name=rpc.operation_name,
+        for url, path_object in self._openapi["paths"].items():
+            for operation_id in self._get_parser("$..operationId").find(path_object):
+                path_item_object = operation_id.context.value
+                rpc = FluentRpc()
+                http = FluentHttp()
+                rpc.operation_name = self._get_external_struct_name(operation_id.value)
+                http.operation_name = self._get_external_struct_name(operation_id.value)
+                if len([m for m in self._api.external_rpc_methods if m.operation_name == rpc.operation_name]) == 0:
+                    self._api.external_rpc_methods.append(rpc)
+                if len([m for m in self._api.internal_http_methods if m.operation_name == http.operation_name]) == 0:
+                    self._api.internal_http_methods.append(http)
+                rpc.request_return_type = "{operation_response_name}Response_StatusCode200".format(
                     operation_response_name=self._get_external_struct_name(rpc.operation_name),
                 )
-            for ref in self._get_parser("$..responses").find(path_item_object):
-                for status_code, schema in ref.value.items():
-                    response = FluentRpcResponse()
-                    response.status_code = status_code
-                    response.schema = schema
-                    rpc.responses.append(response)
+                binary_type = self._get_parser("$..responses..'200'..schema..format").find(path_item_object)
+                ref_type = self._get_parser("$..responses..'200'..'$ref'").find(path_item_object)
+                if len(binary_type) == 1:
+                    rpc.request_return_type = "[]byte"
+                elif len(ref_type) == 1:
+                    request_return_type = self._get_schema_object_name_from_ref(ref_type[0].value)
+                    rpc.request_return_type = self._get_external_struct_name(request_return_type)
+
+                http.request_return_type = rpc.request_return_type
+                ref = self._get_parser("$..requestBody..'$ref'").find(path_item_object)
+                if len(ref) == 1:
+                    new = FluentNew()
+                    new.schema_name = self._get_schema_object_name_from_ref(ref[0].value)
+                    new.schema_object = self._get_schema_object_from_ref(ref[0].value)
+                    new.interface = self._get_external_struct_name(new.schema_name)
+                    new.struct = self._get_internal_name(new.schema_name)
+                    new.method = """New{interface}() {interface}""".format(interface=new.interface)
+                    if len([m for m in self._api.external_new_methods if m.schema_name == new.schema_name]) == 0:
+                        self._api.external_new_methods.append(new)
+                    rpc.request = "{pb_pkg_name}.{operation_name}Request{{{interface}: {struct}.msg()}}".format(
+                        pb_pkg_name=self._protobuf_package_name,
+                        operation_name=rpc.operation_name,
+                        interface=new.interface,
+                        struct=new.struct,
+                    )
+                    rpc.method = """{operation_name}({struct} {interface}) ({request_return_type}, error)""".format(
+                        operation_name=rpc.operation_name,
+                        operation_response_name=self._get_external_struct_name(rpc.operation_name),
+                        struct=new.struct,
+                        interface=new.interface,
+                        request_return_type=rpc.request_return_type,
+                    )
+                    rpc.http_call = """return api.http{operation_name}({struct})""".format(
+                        operation_name=rpc.operation_name,
+                        struct=new.struct,
+                    )
+                    if url.startswith("/"):
+                        url = url[1:]
+                    http.request = """api.httpSendRecv("{url}", {struct}.ToJson(), "{method}")""".format(
+                        operation_name=http.operation_name, url=url, struct=new.struct, method=str(operation_id.context.path.fields[0]).upper()
+                    )
+                    http.method = """http{rpc_method}""".format(rpc_method=rpc.method)
+                else:
+                    rpc.method = """{operation_name}() ({request_return_type}, error)""".format(
+                        operation_name=rpc.operation_name,
+                        operation_response_name=self._get_external_struct_name(rpc.operation_name),
+                        request_return_type=rpc.request_return_type,
+                    )
+                    rpc.http_call = """return api.http{operation_name}()""".format(
+                        operation_name=rpc.operation_name,
+                    )
+                    http.request = """api.httpSendRecv("{url}", "", "{method}")""".format(url=url, method=str(operation_id.context.path.fields[0]).upper())
+                    http.method = """http{rpc_method}""".format(rpc_method=rpc.method)
+                for ref in self._get_parser("$..responses").find(path_item_object):
+                    for status_code, response_object in ref.value.items():
+                        response = FluentRpcResponse()
+                        response.status_code = status_code
+                        if "$ref" in response_object:
+                            response.schema = response_object
+                        else:
+                            response.schema = self._get_parser("$..schema").find(response_object)[0].value
+                        rpc.responses.append(response)
+                        http.responses.append(response)
+
+        self._build_response_interfaces()
 
         # write the go code
         self._write(
             """type {internal_struct_name} struct {{
                 api
                 grpcClient {pb_pkg_name}.OpenapiClient
+                httpClient httpClient
             }}
 
             // grpcConnect builds up a grpc connection
@@ -287,12 +348,49 @@ class OpenApiArtGo(OpenApiArtPlugin):
             }}
 
             // NewApi returns a new instance of the top level interface hierarchy
-            func NewApi() *{internal_struct_name} {{
+            func NewApi() {interface} {{
                 api := {internal_struct_name}{{}}
                 return &api
             }}
+            
+            // httpConnect builds up a http connection
+            func (api *{internal_struct_name}) httpConnect() error {{
+                if api.httpClient.client == nil {{
+                    var verify = !api.http.verify
+                    client := httpClient{{
+                        client: &http.Client{{
+                            Transport: &http.Transport{{
+                                TLSClientConfig: &tls.Config{{InsecureSkipVerify: verify}},
+                            }},
+                        }},
+                        ctx: context.Background(),
+                    }}
+                    api.httpClient = client
+                }}
+                return nil
+            }}
+            
+            func (api *{internal_struct_name}) httpSendRecv(urlPath string, jsonBody string, method string) (*http.Response, error) {{
+                err := api.httpConnect()
+                if err != nil {{
+                    return nil, err
+                }}
+                httpClient := api.httpClient
+                var bodyReader = bytes.NewReader([]byte(jsonBody))
+                queryUrl, err := url.Parse(api.http.location)
+                if err != nil {{
+                    return nil, err
+                }}
+                basePath := fmt.Sprintf(urlPath)
+                queryUrl, _ = queryUrl.Parse(basePath)
+                req, _ := http.NewRequest(method, queryUrl.String(), bodyReader)
+                req.Header.Set("Content-Type", "application/json")
+                req = req.WithContext(httpClient.ctx)
+                return httpClient.client.Do(req)
+            }}            
             """.format(
                 internal_struct_name=self._api.internal_struct_name,
+                interface=self._api.external_interface_name,
                 pb_pkg_name=self._protobuf_package_name,
             )
         )
@@ -337,9 +435,20 @@ class OpenApiArtGo(OpenApiArtPlugin):
                     """.format(
                     status_code=response.status_code,
                 )
-            error_handling += 'return nil, fmt.Errorf("Response not implemented")'
+            error_handling += 'return nil, fmt.Errorf("response not implemented")'
+            if rpc.request_return_type == "[]byte":
+                return_value = "resp.GetStatusCode_200().Bytes"
+            else:
+                return_value = "&{struct}{{obj: resp.GetStatusCode_200().{request_return_type}}}".format(
+                    struct=self._get_internal_name(rpc.request_return_type),
+                    request_return_type=rpc.request_return_type,
+                )
             self._write(
                 """func (api *{internal_struct_name}) {method} {{
+                    if api.hasHttpTransport() {{
+                            {http_call}
+                    }}
+                    
                     if err := api.grpcConnect(); err != nil {{
                         return nil, err
                     }}
@@ -351,7 +460,7 @@ class OpenApiArtGo(OpenApiArtPlugin):
                         return nil, err
                     }}
                     if resp.GetStatusCode_200() != nil {{
-                        return &{operation_response_name}ResponseStatusCode200{{obj: resp.GetStatusCode_200()}}, nil
+                        return {return_value}, nil
                     }}
                     {error_handling}
                 }}
@@ -362,12 +471,64 @@ class OpenApiArtGo(OpenApiArtPlugin):
                     operation_name=rpc.operation_name,
                     operation_response_name=self._get_internal_name(rpc.operation_name),
                     error_handling=error_handling,
+                    return_value=return_value,
+                    http_call=rpc.http_call,
+                )
+            )
+
+        for http in self._api.internal_http_methods:
+            error_handling = ""
+            for response in http.responses:
+                if response.status_code.startswith("2"):
+                    continue
+                error_handling += """if rsp.StatusCode == {status_code} {{
+                        return nil, fmt.Errorf(string(bodyBytes))
+                    }}
+                    """.format(
+                    status_code=response.status_code,
+                )
+            error_handling += 'return nil, fmt.Errorf("response not implemented")'
+            if http.request_return_type == "[]byte":
+                success_handling = """return bodyBytes, nil""".format(package_name=self._protobuf_package_name, operation_name=http.operation_name)
+            else:
+                success_handling = """value := {pb_pkg_name}.{external_name}{{}}
+                    dest := {request_return_type}{{obj: &value}}
+                    if err := dest.FromJson(string(bodyBytes)); err != nil {{
+                        return nil, err
+                    }}
+                    return &dest, nil""".format(
+                    request_return_type=self._get_internal_name(http.request_return_type),
+                    pb_pkg_name=self._protobuf_package_name,
+                    external_name=http.request_return_type,
+                )
+            self._write(
+                """func (api *{internal_struct_name}) {method} {{
+                    rsp, err := {request}
+                    if err != nil {{
+                        return nil, err
+                    }}
+                    bodyBytes, err := ioutil.ReadAll(rsp.Body)
+                    defer rsp.Body.Close()
+                    if err != nil {{
+                        return nil, err
+                    }}
+                    if rsp.StatusCode == 200 {{
+                        {success_handling}
+                    }}
+                    {error_handling}
+                }}
+                """.format(
+                    internal_struct_name=self._api.internal_struct_name,
+                    method=http.method,
+                    request=http.request,
+                    success_handling=success_handling,
+                    error_handling=error_handling,
                 )
             )
 
     def _build_request_interfaces(self):
         for new in self._api.external_new_methods:
-            self._build_interface(new)
+            self._write_interface(new)
 
     def _write_component_interfaces(self):
         while True:
@@ -375,16 +536,17 @@ class OpenApiArtGo(OpenApiArtPlugin):
             if len(components) == 0:
                 break
             for component in components:
-                self._build_interface(component)
+                self._write_interface(component)
 
-    def _write_response_interfaces(self):
+    def _build_response_interfaces(self):
         for rpc in self._api.external_rpc_methods:
             for response in rpc.responses:
-                if response.status_code.startswith("2") is False:
-                    continue
                 new = FluentNew()
-                new.schema_object = response.schema
-                new.struct = "{operation_name}ResponseStatusCode{status_code}".format(
+                new.schema_object = {
+                    "type": "object",
+                    "properties": {"response": response.schema},
+                }
+                new.struct = "{operation_name}Response_StatusCode{status_code}".format(
                     operation_name=self._get_internal_name(rpc.operation_name),
                     status_code=response.status_code,
                 )
@@ -392,9 +554,22 @@ class OpenApiArtGo(OpenApiArtPlugin):
                     operation_name=rpc.operation_name,
                     status_code=response.status_code,
                 )
-                self._build_interface(new)
+                new.method = "New{interface}() {interface}".format(
+                    interface=new.interface,
+                )
+                new.schema_name = self._get_external_struct_name(new.interface)
+                new.isRpcResponse = True
+                self._api.external_new_methods.append(new)
 
-    def _build_interface(self, new):
+    def _write_interface(self, new):
+        if new.schema_name in self._api.components:
+            new = self._api.components[new.schema_name]
+        else:
+            self._api.components[new.schema_name] = new
+        if new.generated is True:
+            return
+        else:
+            new.generated = True
         self._write(
             """type {struct} struct {{
                 obj *{pb_pkg_name}.{interface}
@@ -404,22 +579,60 @@ class OpenApiArtGo(OpenApiArtPlugin):
                 return obj.obj
             }}
 
+            func (obj *{struct}) ToPbText() string {{
+                return proto.MarshalTextString(obj.msg())
+            }}
+
+            func (obj *{struct}) FromPbText(value string) error {{
+                return proto.UnmarshalText(value, obj.msg())
+            }}
+
             func (obj *{struct}) ToYaml() string {{
-                data, _ := yaml.Marshal(obj.msg())
+                opts := protojson.MarshalOptions{{
+                    UseProtoNames:   true,
+                    AllowPartial:    true,
+                    EmitUnpopulated: false,
+                }}
+                data, err := opts.Marshal(obj.msg())
+                data, err = yaml.JSONToYAML(data)
+                if err != nil {{
+                    panic(err)
+                }}
                 return string(data)
             }}
 
             func (obj *{struct}) FromYaml(value string) error {{
-                return yaml.Unmarshal([]byte(value), obj.msg())
+                data, err := yaml.YAMLToJSON([]byte(value))
+                if err != nil {{
+                    return err
+                }}
+                opts := protojson.UnmarshalOptions{{
+                    AllowPartial: true,
+                    DiscardUnknown: true,
+                }}
+                return opts.Unmarshal([]byte(data), obj.msg())
             }}
 
             func (obj *{struct}) ToJson() string {{
-                data, _ := json.Marshal(obj.msg())
+                opts := protojson.MarshalOptions{{
+                    UseProtoNames:   true,
+                    AllowPartial:    true,
+                    EmitUnpopulated: false,
+                    Indent:          "  ",
+                }}
+                data, err := opts.Marshal(obj.msg())
+                if err != nil {{
+                    panic(err)
+                }}
                 return string(data)
             }}
 
             func (obj *{struct}) FromJson(value string) error {{
-                return json.Unmarshal([]byte(value), obj.msg())
+                opts := protojson.UnmarshalOptions{{
+                    AllowPartial: true,
+                    DiscardUnknown: true,
+                }}
+                return opts.Unmarshal([]byte(value), obj.msg())
             }}
         """.format(
                 struct=new.struct,
@@ -428,7 +641,14 @@ class OpenApiArtGo(OpenApiArtPlugin):
             )
         )
         self._build_setters_getters(new)
-        interfaces = ["ToYaml() string", "ToJson() string"]
+        interfaces = [
+            "ToPbText() string",
+            "ToYaml() string",
+            "ToJson() string",
+            "FromPbText(value string) error",
+            "FromYaml(value string) error",
+            "FromJson(value string) error",
+        ]
         for field in new.interface_fields:
             interfaces.append(field.getter_method)
             if field.setter_method is not None:
@@ -449,7 +669,6 @@ class OpenApiArtGo(OpenApiArtPlugin):
             self._write_field_getter(new, field)
             self._write_field_setter(new, field)
             self._write_field_adder(new, field)
-        new.generated = True
 
     def _write_field_getter(self, new, field):
         if field.getter_method is None:
@@ -459,8 +678,7 @@ class OpenApiArtGo(OpenApiArtPlugin):
                 body = """if obj.obj.{name} == nil {{
                         obj.obj.{name} = []*{pb_pkg_name}.{pb_struct}{{}}
                     }}
-                    return &{parent}{interface}Iter{{obj: obj}}
-                """.format(
+                    return &{parent}{interface}Iter{{obj: obj}}""".format(
                     name=field.name,
                     pb_pkg_name=self._protobuf_package_name,
                     pb_struct=field.external_struct,
@@ -474,32 +692,84 @@ class OpenApiArtGo(OpenApiArtPlugin):
                     for _, item := range value {{
                         obj.obj.{name} = append(obj.obj.{name}, item)
                     }}
-                    return obj
-                """.format(
+                    return obj""".format(
                     name=field.name,
                     type=field.type,
                 )
         elif field.struct is not None:
-            if field.isPointer:
-                body = """if obj.obj.{name} == nil {{
-                        obj.obj.{name} = &{pb_pkg_name}.{pb_struct}{{}}
-                    }}
-                    return &{struct}{{obj: obj.obj.{name}}}
+            # at this time proto generation ignores the optional keyword
+            # if the type is an object
+            set_choice = ""
+            if field.setChoiceValue is not None:
+                set_choice = """obj.SetChoice({interface}Choice.{enum})""".format(
+                    interface=new.interface,
+                    enum=field.setChoiceValue,
+                )
+            body = """if obj.obj.{name} == nil {{
+                    obj.obj.{name} = &{pb_pkg_name}.{pb_struct}{{}}
+                }}
+                {set_choice}
+                return &{struct}{{obj: obj.obj.{name}}}""".format(
+                name=field.name,
+                pb_pkg_name=self._protobuf_package_name,
+                pb_struct=field.external_struct,
+                struct=field.struct,
+                set_choice=set_choice,
+            )
+        elif field.isEnum:
+            enum_types = []
+            for enum in field.enums:
+                enum_types.append(
+                    "{enumupper} {interface}{fieldname}Enum".format(
+                        enumupper=enum.upper(),
+                        interface=new.interface,
+                        fieldname=field.name,
+                    )
+                )
+            enum_values = []
+            for enum in field.enums:
+                enum_values.append(
+                    '{enumupper}: {interface}{fieldname}Enum("{enum}")'.format(
+                        enumupper=enum.upper(),
+                        interface=new.interface,
+                        fieldname=field.name,
+                        enum=enum,
+                    )
+                )
+            self._write(
+                """type {interface}{fieldname}Enum string
+
+                var {interface}{fieldname} = struct {{
+                    {enum_types}
+                }} {{
+                    {enum_values},
+                }}
                 """.format(
-                    name=field.name,
-                    pb_pkg_name=self._protobuf_package_name,
-                    pb_struct=field.external_struct,
-                    struct=field.struct,
+                    interface=new.interface,
+                    fieldname=field.name,
+                    enum_types="\n".join(enum_types),
+                    enum_values=",\n".join(enum_values),
                 )
-            else:
-                body = "return &{struct}{{obj: obj.obj.{name}}}".format(
-                    struct=field.struct,
-                    name=field.name,
+            )
+            self._write(
+                """func (obj *{struct}) {fieldname}() {interface}{fieldname}Enum {{
+                return {interface}{fieldname}Enum(obj.obj.{fieldname}.Enum().String())
+            }}
+            """.format(
+                    struct=new.struct,
+                    interface=new.interface,
+                    fieldname=field.name,
                 )
+            )
+            return
         elif field.isPointer:
-            body = """return *obj.obj.{name}""".format(name=field.name)
+            body = """return *obj.obj.{fieldname}""".format(
+                fieldname=field.name,
+            )
         else:
-            body = """return obj.obj.{name}""".format(name=field.name)
+            body = """return obj.obj.{fieldname}""".format(
+                fieldname=field.name,
+            )
         self._write(
             """
             // {fieldname} returns a {fieldtype}\n{description}
@@ -507,7 +777,7 @@ class OpenApiArtGo(OpenApiArtPlugin):
                 {body}
             }}
             """.format(
-                fieldname=field.name,
+                fieldname=self._get_external_struct_name(field.name),
                 struct=new.struct,
                 getter_method=field.getter_method,
                 body=body,
@@ -532,25 +802,62 @@ class OpenApiArtGo(OpenApiArtPlugin):
                 fieldtype=field.type,
             )
             return
+        elif field.isEnum:
+            if field.isPointer:
+                body = """enumValue := {pb_pkg_name}.{interface}_{fieldname}_Enum(intValue)
+                obj.obj.{fieldname} = &enumValue""".format(
+                    pb_pkg_name=self._protobuf_package_name,
+                    interface=new.interface,
+                    fieldname=field.name,
+                )
+            else:
+                body = """obj.obj.{fieldname} = {pb_pkg_name}.{interface}_{fieldname}_Enum(intValue)""".format(
+                    pb_pkg_name=self._protobuf_package_name,
+                    interface=new.interface,
+                    fieldname=field.name,
+                )
+            self._write(
+                """func (obj* {struct}) Set{fieldname}(value {interface}{fieldname}Enum) {interface} {{
+                intValue := {pb_pkg_name}.{interface}_{fieldname}_Enum_value[string(value)]
+                {body}
+                return obj
+            }}
+            """.format(
+                    pb_pkg_name=self._protobuf_package_name,
+                    interface=new.interface,
+                    struct=new.struct,
+                    fieldname=field.name,
+                    body=body,
+                )
+            )
+            return
         elif field.isPointer:
             body = """obj.obj.{fieldname} = &value""".format(fieldname=field.name)
         else:
             body = """obj.obj.{fieldname} = value""".format(fieldname=field.name)
+        set_choice = ""
+        if field.setChoiceValue is not None:
+            set_choice = """obj.SetChoice({interface}Choice.{enum})""".format(
+                interface=new.interface,
+                enum=field.setChoiceValue,
+            )
         self._write(
             """
             // Set{fieldname} sets the {fieldtype} value in the {fieldstruct} object\n{description}
             func (obj *{newstruct}) {setter_method} {{
                 {body}
+                {set_choice}
                 return obj
             }}
             """.format(
-                fieldname=field.name,
+                fieldname=self._get_external_struct_name(field.name),
                 newstruct=new.struct,
                 setter_method=field.setter_method,
                 body=body,
                 description=field.description,
                 fieldtype=field.type,
-                fieldstruct=field.external_struct,
+                fieldstruct=new.interface,
+                set_choice=set_choice,
             )
         )
 
@@ -606,25 +913,52 @@ class OpenApiArtGo(OpenApiArtPlugin):
         """Add new FluentField objects for each interface field"""
         if "properties" not in fluent_new.schema_object:
             return
+        choice_enums = self._get_parser("$..choice..enum").find(fluent_new.schema_object["properties"])
         for property_name, property_schema in fluent_new.schema_object["properties"].items():
-            if len(self._get_parser("$..enum").find(property_schema)) > 0:  # temporary
+            if len(self._get_parser("$.items.enum").find(property_schema)) > 0:  # temporary
                 continue
             field = FluentField()
             field.schema = property_schema
             field.description = self._get_description(property_schema)
             field.name = self._get_external_field_name(property_name)
             field.type = self._get_struct_field_type(property_schema)
+            if len(choice_enums) == 1 and property_name in choice_enums[0].value:
+                field.setChoiceValue = property_name.upper()
+            else:
+                field.setChoiceValue = None
+            field.isEnum = len(self._get_parser("$..enum").find(property_schema)) > 0
+            if field.isEnum:
+                field.enums = self._get_parser("$..enum").find(property_schema)[0].value
+                if "unspecified" in field.enums:
+                    field.enums.remove("unspecified")
+            if fluent_new.isRpcResponse:
+                if field.type == "[]byte":
+                    field.name = "Bytes"
+                elif "$ref" in property_schema:
+                    schema_name = self._get_schema_object_name_from_ref(property_schema["$ref"])
+                    field.name = self._get_external_struct_name(schema_name)
             field.isOptional = fluent_new.isOptional(property_name)
             field.isPointer = False if field.type.startswith("[") else field.isOptional
-            field.getter_method = "{name}() {ftype}".format(
-                name=self._get_external_struct_name(field.name),
-                ftype=field.type,
-            )
+            if field.isEnum:
+                field.getter_method = "{fieldname}() {interface}{fieldname}Enum".format(
+                    fieldname=self._get_external_struct_name(field.name),
+                    interface=fluent_new.interface,
+                )
+            else:
+                field.getter_method = "{name}() {ftype}".format(
+                    name=self._get_external_struct_name(field.name),
+                    ftype=field.type,
+                )
             if "$ref" in property_schema:
                 schema_name = self._get_schema_object_name_from_ref(property_schema["$ref"])
                 field.struct = self._get_internal_name(schema_name)
                 field.external_struct = self._get_external_struct_name(schema_name)
-            if field.type in self._oapi_go_types.values():
+            if field.isEnum:
+                field.setter_method = "Set{fieldname}(value {interface}{fieldname}Enum) {interface}".format(
+                    fieldname=self._get_external_struct_name(field.name),
+                    interface=fluent_new.interface,
+                )
+            elif field.type in self._oapi_go_types.values():
                 field.setter_method = "Set{name}(value {ftype}) {interface}".format(
                     name=self._get_external_struct_name(field.name),
                     ftype=field.type,
